import { GraphQLSchema } from 'graphql';
import { IRange as GraphQLRange, IPosition as GraphQLPosition, Diagnostic, ContextToken } from 'graphql-language-service';
import type * as monaco from 'monaco-editor';
import type { EnrichedLanguageService } from './EnrichedLanguageService';
export declare type BridgeOptions = {
    schema: GraphQLSchema;
    document: string;
    position: GraphQLPosition;
    model: monaco.editor.ITextModel;
    token: ContextToken;
    languageService: EnrichedLanguageService;
};
export declare type HoverSource = {
    forNode(options: BridgeOptions): string | null | Promise<string | null>;
};
export declare type KeyboardShortcut = {
    keybinding: number;
    handler: monaco.editor.ICommandHandler;
};
export declare type DiagnosticsSource = {
    forDocument(options: Pick<BridgeOptions, 'document' | 'languageService' | 'model'>): monaco.editor.IMarkerData[] | null | Promise<monaco.editor.IMarkerData[] | null>;
};
export declare const coreDiagnosticsSource: DiagnosticsSource;
export declare type DecorationsSource = {
    forDocument(options: Pick<BridgeOptions, 'document' | 'languageService' | 'model'> & {
        editor: monaco.editor.IStandaloneCodeEditor;
        monaco: typeof monaco;
    }): void | Promise<void>;
};
export declare type DefinitionSource = {
    forNode(options: BridgeOptions): monaco.languages.Definition[] | null | Promise<monaco.languages.Definition[] | null>;
};
export declare const coreDefinitionSource: DefinitionSource;
export declare const coreHoverSource: HoverSource;
export declare const debugHoverSource: HoverSource;
export declare function toGraphQLPosition(position: monaco.Position): GraphQLPosition;
export declare function toMarkerData(diagnostic: Diagnostic): monaco.editor.IMarkerData;
export declare function toMonacoRange(range: GraphQLRange): monaco.IRange;
export declare type PreviewAction = {
    id: string;
    title: string;
    onOpen: () => HTMLElement;
    onClose: () => void;
};
export declare type EditorAction = {
    id: string;
    label: string;
    keybindings?: number[];
    contextMenuGroupId?: string;
    contextMenuOrder?: number;
    onRun: (options: {
        editor: monaco.editor.IStandaloneCodeEditor;
        monaco: typeof monaco;
        bridge: BridgeOptions;
    }) => void;
};
export declare function showWidgetInPosition(editorInstance: monaco.editor.IStandaloneCodeEditor, position: BridgeOptions['position'], htmlElement: HTMLElement): void;
