import * as React from 'react';
import MonacoEditor from '@monaco-editor/react';
import { getHoverInformation, Position, DIAGNOSTIC_SEVERITY, getRange, LanguageService, getTokenAtPosition } from 'graphql-language-service';

function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) {
  try {
    var info = gen[key](arg);
    var value = info.value;
  } catch (error) {
    reject(error);
    return;
  }

  if (info.done) {
    resolve(value);
  } else {
    Promise.resolve(value).then(_next, _throw);
  }
}

function _asyncToGenerator(fn) {
  return function () {
    var self = this,
        args = arguments;
    return new Promise(function (resolve, reject) {
      var gen = fn.apply(self, args);

      function _next(value) {
        asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value);
      }

      function _throw(err) {
        asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err);
      }

      _next(undefined);
    });
  };
}

function _classCallCheck(instance, Constructor) {
  if (!(instance instanceof Constructor)) {
    throw new TypeError("Cannot call a class as a function");
  }
}

function _defineProperties(target, props) {
  for (var i = 0; i < props.length; i++) {
    var descriptor = props[i];
    descriptor.enumerable = descriptor.enumerable || false;
    descriptor.configurable = true;
    if ("value" in descriptor) descriptor.writable = true;
    Object.defineProperty(target, descriptor.key, descriptor);
  }
}

function _createClass(Constructor, protoProps, staticProps) {
  if (protoProps) _defineProperties(Constructor.prototype, protoProps);
  if (staticProps) _defineProperties(Constructor, staticProps);
  return Constructor;
}

function _defineProperty(obj, key, value) {
  if (key in obj) {
    Object.defineProperty(obj, key, {
      value: value,
      enumerable: true,
      configurable: true,
      writable: true
    });
  } else {
    obj[key] = value;
  }

  return obj;
}

function _extends() {
  _extends = Object.assign || function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };

  return _extends.apply(this, arguments);
}

function ownKeys(object, enumerableOnly) {
  var keys = Object.keys(object);

  if (Object.getOwnPropertySymbols) {
    var symbols = Object.getOwnPropertySymbols(object);
    if (enumerableOnly) symbols = symbols.filter(function (sym) {
      return Object.getOwnPropertyDescriptor(object, sym).enumerable;
    });
    keys.push.apply(keys, symbols);
  }

  return keys;
}

function _objectSpread2(target) {
  for (var i = 1; i < arguments.length; i++) {
    var source = arguments[i] != null ? arguments[i] : {};

    if (i % 2) {
      ownKeys(Object(source), true).forEach(function (key) {
        _defineProperty(target, key, source[key]);
      });
    } else if (Object.getOwnPropertyDescriptors) {
      Object.defineProperties(target, Object.getOwnPropertyDescriptors(source));
    } else {
      ownKeys(Object(source)).forEach(function (key) {
        Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key));
      });
    }
  }

  return target;
}

function _inherits(subClass, superClass) {
  if (typeof superClass !== "function" && superClass !== null) {
    throw new TypeError("Super expression must either be null or a function");
  }

  subClass.prototype = Object.create(superClass && superClass.prototype, {
    constructor: {
      value: subClass,
      writable: true,
      configurable: true
    }
  });
  if (superClass) _setPrototypeOf(subClass, superClass);
}

function _getPrototypeOf(o) {
  _getPrototypeOf = Object.setPrototypeOf ? Object.getPrototypeOf : function _getPrototypeOf(o) {
    return o.__proto__ || Object.getPrototypeOf(o);
  };
  return _getPrototypeOf(o);
}

function _setPrototypeOf(o, p) {
  _setPrototypeOf = Object.setPrototypeOf || function _setPrototypeOf(o, p) {
    o.__proto__ = p;
    return o;
  };

  return _setPrototypeOf(o, p);
}

function _isNativeReflectConstruct() {
  if (typeof Reflect === "undefined" || !Reflect.construct) return false;
  if (Reflect.construct.sham) return false;
  if (typeof Proxy === "function") return true;

  try {
    Boolean.prototype.valueOf.call(Reflect.construct(Boolean, [], function () {}));
    return true;
  } catch (e) {
    return false;
  }
}

function _objectWithoutPropertiesLoose(source, excluded) {
  if (source == null) return {};
  var target = {};
  var sourceKeys = Object.keys(source);
  var key, i;

  for (i = 0; i < sourceKeys.length; i++) {
    key = sourceKeys[i];
    if (excluded.indexOf(key) >= 0) continue;
    target[key] = source[key];
  }

  return target;
}

function _objectWithoutProperties(source, excluded) {
  if (source == null) return {};

  var target = _objectWithoutPropertiesLoose(source, excluded);

  var key, i;

  if (Object.getOwnPropertySymbols) {
    var sourceSymbolKeys = Object.getOwnPropertySymbols(source);

    for (i = 0; i < sourceSymbolKeys.length; i++) {
      key = sourceSymbolKeys[i];
      if (excluded.indexOf(key) >= 0) continue;
      if (!Object.prototype.propertyIsEnumerable.call(source, key)) continue;
      target[key] = source[key];
    }
  }

  return target;
}

function _assertThisInitialized(self) {
  if (self === void 0) {
    throw new ReferenceError("this hasn't been initialised - super() hasn't been called");
  }

  return self;
}

function _possibleConstructorReturn(self, call) {
  if (call && (typeof call === "object" || typeof call === "function")) {
    return call;
  }

  return _assertThisInitialized(self);
}

function _createSuper(Derived) {
  var hasNativeReflectConstruct = _isNativeReflectConstruct();

  return function _createSuperInternal() {
    var Super = _getPrototypeOf(Derived),
        result;

    if (hasNativeReflectConstruct) {
      var NewTarget = _getPrototypeOf(this).constructor;

      result = Reflect.construct(Super, arguments, NewTarget);
    } else {
      result = Super.apply(this, arguments);
    }

    return _possibleConstructorReturn(this, result);
  };
}

function _slicedToArray(arr, i) {
  return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _unsupportedIterableToArray(arr, i) || _nonIterableRest();
}

function _toConsumableArray(arr) {
  return _arrayWithoutHoles(arr) || _iterableToArray(arr) || _unsupportedIterableToArray(arr) || _nonIterableSpread();
}

function _arrayWithoutHoles(arr) {
  if (Array.isArray(arr)) return _arrayLikeToArray(arr);
}

function _arrayWithHoles(arr) {
  if (Array.isArray(arr)) return arr;
}

function _iterableToArray(iter) {
  if (typeof Symbol !== "undefined" && iter[Symbol.iterator] != null || iter["@@iterator"] != null) return Array.from(iter);
}

function _iterableToArrayLimit(arr, i) {
  var _i = arr && (typeof Symbol !== "undefined" && arr[Symbol.iterator] || arr["@@iterator"]);

  if (_i == null) return;
  var _arr = [];
  var _n = true;
  var _d = false;

  var _s, _e;

  try {
    for (_i = _i.call(arr); !(_n = (_s = _i.next()).done); _n = true) {
      _arr.push(_s.value);

      if (i && _arr.length === i) break;
    }
  } catch (err) {
    _d = true;
    _e = err;
  } finally {
    try {
      if (!_n && _i["return"] != null) _i["return"]();
    } finally {
      if (_d) throw _e;
    }
  }

  return _arr;
}

function _unsupportedIterableToArray(o, minLen) {
  if (!o) return;
  if (typeof o === "string") return _arrayLikeToArray(o, minLen);
  var n = Object.prototype.toString.call(o).slice(8, -1);
  if (n === "Object" && o.constructor) n = o.constructor.name;
  if (n === "Map" || n === "Set") return Array.from(o);
  if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen);
}

function _arrayLikeToArray(arr, len) {
  if (len == null || len > arr.length) len = arr.length;

  for (var i = 0, arr2 = new Array(len); i < len; i++) arr2[i] = arr[i];

  return arr2;
}

function _nonIterableSpread() {
  throw new TypeError("Invalid attempt to spread non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.");
}

function _nonIterableRest() {
  throw new TypeError("Invalid attempt to destructure non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.");
}

function _createForOfIteratorHelper(o, allowArrayLike) {
  var it = typeof Symbol !== "undefined" && o[Symbol.iterator] || o["@@iterator"];

  if (!it) {
    if (Array.isArray(o) || (it = _unsupportedIterableToArray(o)) || allowArrayLike && o && typeof o.length === "number") {
      if (it) o = it;
      var i = 0;

      var F = function () {};

      return {
        s: F,
        n: function () {
          if (i >= o.length) return {
            done: true
          };
          return {
            done: false,
            value: o[i++]
          };
        },
        e: function (e) {
          throw e;
        },
        f: F
      };
    }

    throw new TypeError("Invalid attempt to iterate non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.");
  }

  var normalCompletion = true,
      didErr = false,
      err;
  return {
    s: function () {
      it = it.call(o);
    },
    n: function () {
      var step = it.next();
      normalCompletion = step.done;
      return step;
    },
    e: function (e) {
      didErr = true;
      err = e;
    },
    f: function () {
      try {
        if (!normalCompletion && it.return != null) it.return();
      } finally {
        if (didErr) throw err;
      }
    }
  };
}

var coreDiagnosticsSource = {
  forDocument: function forDocument(_ref) {
    return _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee() {
      var model, document, languageService;
      return regeneratorRuntime.wrap(function _callee$(_context) {
        while (1) {
          switch (_context.prev = _context.next) {
            case 0:
              model = _ref.model, document = _ref.document, languageService = _ref.languageService;
              return _context.abrupt("return", languageService.getDiagnostics(model.uri.toString(), document).then(function (diag) {
                return diag.map(toMarkerData);
              }).catch(function (e) {
                if ('message' in e && 'locations' in e) {
                  return [toMarkerData({
                    severity: DIAGNOSTIC_SEVERITY.Error,
                    message: e.message,
                    source: 'GraphQL: Syntax',
                    range: getRange(e.locations[0], document)
                  })];
                } else {
                  console.warn("GraphQL getDiagnostics failed unexpected error: ", e);
                  return [];
                }
              }));

            case 2:
            case "end":
              return _context.stop();
          }
        }
      }, _callee);
    }))();
  }
};
var coreDefinitionSource = {
  forNode: function forNode(_ref2) {
    var schema = _ref2.schema,
        model = _ref2.model,
        token = _ref2.token;

    if (token.state && token.state.kind === 'NamedType' && token.state.name) {
      var type = schema.getType(token.state.name);

      if (type && type.astNode && type.astNode.loc) {
        return [{
          range: {
            startLineNumber: type.astNode.loc.startToken.line,
            startColumn: type.astNode.loc.startToken.column,
            endLineNumber: type.astNode.loc.endToken.line + 1,
            endColumn: type.astNode.loc.endToken.column
          },
          uri: model.uri
        }];
      }
    }

    return [];
  }
};
var coreHoverSource = {
  forNode: function forNode(_ref3) {
    var schema = _ref3.schema,
        document = _ref3.document,
        position = _ref3.position,
        token = _ref3.token;
    return getHoverInformation(schema, document, position, token);
  }
};
var debugHoverSource = {
  forNode: function forNode(_ref4) {
    var token = _ref4.token;
    return '```json\n' + JSON.stringify(token.state, null, 2) + '\n```';
  }
};
function toGraphQLPosition(position) {
  return new Position(position.lineNumber - 1, position.column - 1);
}
function toMarkerData(diagnostic) {
  return {
    startLineNumber: diagnostic.range.start.line + 1,
    endLineNumber: diagnostic.range.end.line + 1,
    startColumn: diagnostic.range.start.character + 1,
    endColumn: diagnostic.range.end.character,
    message: diagnostic.message,
    severity: 5,
    // severity: toMonacoSeverity(diagnostic.severity),
    code: diagnostic.code || undefined
  };
}
function toMonacoRange(range) {
  return {
    startLineNumber: range.start.line + 1,
    startColumn: range.start.character + 1,
    endLineNumber: range.end.line + 1,
    endColumn: range.end.character + 1
  };
}
function showWidgetInPosition(editorInstance, position, htmlElement) {
  editorInstance.changeViewZones(function (changeAccessor) {
    changeAccessor.addZone({
      afterLineNumber: position.line + 1,
      heightInPx: 60,
      domNode: htmlElement
    });
  });
}

var EnrichedLanguageService = /*#__PURE__*/function (_LanguageService) {
  _inherits(EnrichedLanguageService, _LanguageService);

  var _super = _createSuper(EnrichedLanguageService);

  function EnrichedLanguageService() {
    _classCallCheck(this, EnrichedLanguageService);

    return _super.apply(this, arguments);
  }

  _createClass(EnrichedLanguageService, [{
    key: "getNodeAtPosition",
    value: function () {
      var _getNodeAtPosition = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee(schema, document, position) {
        var token;
        return regeneratorRuntime.wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                if (!schema) {
                  _context.next = 4;
                  break;
                }

                token = getTokenAtPosition(document, position);

                if (!token) {
                  _context.next = 4;
                  break;
                }

                return _context.abrupt("return", token);

              case 4:
                return _context.abrupt("return", null);

              case 5:
              case "end":
                return _context.stop();
            }
          }
        }, _callee);
      }));

      function getNodeAtPosition(_x, _x2, _x3) {
        return _getNodeAtPosition.apply(this, arguments);
      }

      return getNodeAtPosition;
    }()
  }, {
    key: "buildBridgeForProviders",
    value: function () {
      var _buildBridgeForProviders = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee2(model, position) {
        var graphQLPosition, document, schema, tokenAtPosition;
        return regeneratorRuntime.wrap(function _callee2$(_context2) {
          while (1) {
            switch (_context2.prev = _context2.next) {
              case 0:
                graphQLPosition = toGraphQLPosition(position);
                document = model.getValue();
                _context2.next = 4;
                return this.getSchema();

              case 4:
                schema = _context2.sent;

                if (schema) {
                  _context2.next = 7;
                  break;
                }

                return _context2.abrupt("return", null);

              case 7:
                _context2.next = 9;
                return this.getNodeAtPosition(schema, document, graphQLPosition);

              case 9:
                tokenAtPosition = _context2.sent;

                if (tokenAtPosition) {
                  _context2.next = 12;
                  break;
                }

                return _context2.abrupt("return", null);

              case 12:
                return _context2.abrupt("return", {
                  languageService: this,
                  position: graphQLPosition,
                  document: document,
                  schema: schema,
                  token: tokenAtPosition,
                  model: model
                });

              case 13:
              case "end":
                return _context2.stop();
            }
          }
        }, _callee2, this);
      }));

      function buildBridgeForProviders(_x4, _x5) {
        return _buildBridgeForProviders.apply(this, arguments);
      }

      return buildBridgeForProviders;
    }()
  }, {
    key: "getDefinitionProvider",
    value: function getDefinitionProvider(rawSources) {
      var _this = this;

      var sources = [].concat(_toConsumableArray(rawSources), [coreDefinitionSource]);
      return {
        provideDefinition: function () {
          var _provideDefinition = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee3(model, position) {
            var _ref;

            var bridge, nestedArrays, items;
            return regeneratorRuntime.wrap(function _callee3$(_context3) {
              while (1) {
                switch (_context3.prev = _context3.next) {
                  case 0:
                    _context3.next = 2;
                    return _this.buildBridgeForProviders(model, position);

                  case 2:
                    bridge = _context3.sent;

                    if (bridge) {
                      _context3.next = 5;
                      break;
                    }

                    return _context3.abrupt("return", []);

                  case 5:
                    _context3.next = 7;
                    return Promise.all(sources.map(function (source) {
                      return source.forNode(bridge);
                    }));

                  case 7:
                    nestedArrays = _context3.sent.filter(Boolean);
                    items = (_ref = []).concat.apply(_ref, _toConsumableArray(nestedArrays));
                    return _context3.abrupt("return", items);

                  case 10:
                  case "end":
                    return _context3.stop();
                }
              }
            }, _callee3);
          }));

          function provideDefinition(_x6, _x7) {
            return _provideDefinition.apply(this, arguments);
          }

          return provideDefinition;
        }()
      };
    }
  }, {
    key: "getHoverProvider",
    value: function getHoverProvider(rawSources) {
      var _this2 = this;

      var sources = [].concat(_toConsumableArray(rawSources), [coreHoverSource]);
      return {
        provideHover: function () {
          var _provideHover = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee5(model, position) {
            var info, contents;
            return regeneratorRuntime.wrap(function _callee5$(_context5) {
              while (1) {
                switch (_context5.prev = _context5.next) {
                  case 0:
                    _context5.next = 2;
                    return _this2.buildBridgeForProviders(model, position);

                  case 2:
                    info = _context5.sent;

                    if (info) {
                      _context5.next = 5;
                      break;
                    }

                    return _context5.abrupt("return");

                  case 5:
                    _context5.next = 7;
                    return Promise.all(sources.map( /*#__PURE__*/function () {
                      var _ref2 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee4(source) {
                        var value;
                        return regeneratorRuntime.wrap(function _callee4$(_context4) {
                          while (1) {
                            switch (_context4.prev = _context4.next) {
                              case 0:
                                _context4.next = 2;
                                return source.forNode(info);

                              case 2:
                                value = _context4.sent;

                                if (value) {
                                  _context4.next = 5;
                                  break;
                                }

                                return _context4.abrupt("return", null);

                              case 5:
                                return _context4.abrupt("return", {
                                  value: value
                                });

                              case 6:
                              case "end":
                                return _context4.stop();
                            }
                          }
                        }, _callee4);
                      }));

                      return function (_x10) {
                        return _ref2.apply(this, arguments);
                      };
                    }()));

                  case 7:
                    contents = _context5.sent.filter(Boolean);
                    return _context5.abrupt("return", {
                      contents: contents,
                      range: toMonacoRange(getRange({
                        column: info.position.character,
                        line: info.position.line + 1
                      }, info.document))
                    });

                  case 9:
                  case "end":
                    return _context5.stop();
                }
              }
            }, _callee5);
          }));

          function provideHover(_x8, _x9) {
            return _provideHover.apply(this, arguments);
          }

          return provideHover;
        }()
      };
    }
  }, {
    key: "handleDecorations",
    value: function () {
      var _handleDecorations = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee6(decorationSources, model, monacoInstance, editorInstance) {
        var _iterator, _step, source;

        return regeneratorRuntime.wrap(function _callee6$(_context6) {
          while (1) {
            switch (_context6.prev = _context6.next) {
              case 0:
                _iterator = _createForOfIteratorHelper(decorationSources);

                try {
                  for (_iterator.s(); !(_step = _iterator.n()).done;) {
                    source = _step.value;
                    source.forDocument({
                      monaco: monacoInstance,
                      editor: editorInstance,
                      document: model.getValue(),
                      model: model,
                      languageService: this
                    });
                  }
                } catch (err) {
                  _iterator.e(err);
                } finally {
                  _iterator.f();
                }

              case 2:
              case "end":
                return _context6.stop();
            }
          }
        }, _callee6, this);
      }));

      function handleDecorations(_x11, _x12, _x13, _x14) {
        return _handleDecorations.apply(this, arguments);
      }

      return handleDecorations;
    }()
  }, {
    key: "handleDiagnostics",
    value: function () {
      var _handleDiagnostics = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee7(rawDiagnosticsSources, model, monacoInstance) {
        var _this3 = this,
            _ref3;

        var diagnosticsSources, nestedArrays, markerData;
        return regeneratorRuntime.wrap(function _callee7$(_context7) {
          while (1) {
            switch (_context7.prev = _context7.next) {
              case 0:
                diagnosticsSources = [].concat(_toConsumableArray(rawDiagnosticsSources), [coreDiagnosticsSource]);
                _context7.next = 3;
                return Promise.all(diagnosticsSources.map(function (source) {
                  return source.forDocument({
                    languageService: _this3,
                    model: model,
                    document: model.getValue().toString()
                  });
                }));

              case 3:
                nestedArrays = _context7.sent.filter(Boolean);
                markerData = (_ref3 = []).concat.apply(_ref3, _toConsumableArray(nestedArrays));

                if (markerData.length) {
                  monacoInstance.editor.setModelMarkers(model, 'graphql', markerData);
                }

              case 6:
              case "end":
                return _context7.stop();
            }
          }
        }, _callee7);
      }));

      function handleDiagnostics(_x15, _x16, _x17) {
        return _handleDiagnostics.apply(this, arguments);
      }

      return handleDiagnostics;
    }()
  }, {
    key: "getModelChangeHandler",
    value: function getModelChangeHandler() {
      var _this4 = this;

      return /*#__PURE__*/function () {
        var _ref4 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee8(editorInstance, monacoInstance, diagnosticsSources, decorationsSources) {
          var model;
          return regeneratorRuntime.wrap(function _callee8$(_context8) {
            while (1) {
              switch (_context8.prev = _context8.next) {
                case 0:
                  model = editorInstance.getModel();

                  if (model) {
                    _context8.next = 3;
                    break;
                  }

                  return _context8.abrupt("return");

                case 3:
                  _context8.next = 5;
                  return Promise.all([_this4.handleDiagnostics(diagnosticsSources, model, monacoInstance), _this4.handleDecorations(decorationsSources, model, monacoInstance, editorInstance)]);

                case 5:
                case "end":
                  return _context8.stop();
              }
            }
          }, _callee8);
        }));

        return function (_x18, _x19, _x20, _x21) {
          return _ref4.apply(this, arguments);
        };
      }();
    }
  }, {
    key: "trySchema",
    value: function trySchema(sdl) {
      // XX-DOTAN: In case of an issue with the schema parsing, we'll get the error through
      // the Marker, and we can safely ignore it here.
      // eslint-disable-next-line @typescript-eslint/no-empty-function
      this.setSchema(sdl).catch(function () {});
    }
  }]);

  return EnrichedLanguageService;
}(LanguageService);

var SchemaEditor = function SchemaEditor(_ref) {
  var schema = _ref.schema,
      _ref$hoverProviders = _ref.hoverProviders,
      hoverProviders = _ref$hoverProviders === void 0 ? [] : _ref$hoverProviders,
      _ref$definitionProvid = _ref.definitionProviders,
      definitionProviders = _ref$definitionProvid === void 0 ? [] : _ref$definitionProvid,
      _ref$diagnosticsProvi = _ref.diagnosticsProviders,
      diagnosticsProviders = _ref$diagnosticsProvi === void 0 ? [] : _ref$diagnosticsProvi,
      _ref$decorationsProvi = _ref.decorationsProviders,
      decorationsProviders = _ref$decorationsProvi === void 0 ? [] : _ref$decorationsProvi,
      _ref$actions = _ref.actions,
      actions = _ref$actions === void 0 ? [] : _ref$actions,
      _ref$keyboardShortcut = _ref.keyboardShortcuts,
      keyboardShortcuts = _ref$keyboardShortcut === void 0 ? {} : _ref$keyboardShortcut,
      rest = _objectWithoutProperties(_ref, ["schema", "hoverProviders", "definitionProviders", "diagnosticsProviders", "decorationsProviders", "actions", "keyboardShortcuts"]);

  var _React$useState = React.useState(null),
      _React$useState2 = _slicedToArray(_React$useState, 2),
      editorRef = _React$useState2[0],
      setEditor = _React$useState2[1];

  var _React$useState3 = React.useState(null),
      _React$useState4 = _slicedToArray(_React$useState3, 2),
      monacoRef = _React$useState4[0],
      setMonaco = _React$useState4[1];

  var languageService = React.useMemo(function () {
    return new EnrichedLanguageService({
      schemaString: schema,
      schemaConfig: {
        buildSchemaOptions: {
          assumeValid: true,
          assumeValidSDL: true
        }
      }
    });
  }, []);
  React.useEffect(function () {
    if (monacoRef && editorRef) {
      for (var _i = 0, _Object$entries = Object.entries(keyboardShortcuts); _i < _Object$entries.length; _i++) {
        var _Object$entries$_i = _slicedToArray(_Object$entries[_i], 2),
            name = _Object$entries$_i[0],
            action = _Object$entries$_i[1];

        editorRef.addCommand(action.keybinding, action.handler, name);
      }

      var _iterator = _createForOfIteratorHelper(actions),
          _step;

      try {
        var _loop = function _loop() {
          var action = _step.value;
          editorRef.addAction({
            id: action.id,
            label: action.label,
            keybindings: action.keybindings,
            contextMenuGroupId: action.contextMenuGroupId || 'navigation',
            contextMenuOrder: action.contextMenuOrder,
            run: function () {
              var _run = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee(editor) {
                var model, position, bridge;
                return regeneratorRuntime.wrap(function _callee$(_context) {
                  while (1) {
                    switch (_context.prev = _context.next) {
                      case 0:
                        model = editor.getModel();
                        position = editor.getPosition();

                        if (!(model && position)) {
                          _context.next = 7;
                          break;
                        }

                        _context.next = 5;
                        return languageService.buildBridgeForProviders(model, position);

                      case 5:
                        bridge = _context.sent;

                        if (bridge) {
                          action.onRun({
                            editor: editorRef,
                            monaco: monacoRef,
                            bridge: bridge
                          });
                        }

                      case 7:
                      case "end":
                        return _context.stop();
                    }
                  }
                }, _callee);
              }));

              function run(_x) {
                return _run.apply(this, arguments);
              }

              return run;
            }()
          });
        };

        for (_iterator.s(); !(_step = _iterator.n()).done;) {
          _loop();
        }
      } catch (err) {
        _iterator.e(err);
      } finally {
        _iterator.f();
      }

      var handler = languageService.getModelChangeHandler();
      handler(editorRef, monacoRef, diagnosticsProviders, decorationsProviders);
      var onChangeDisposable = editorRef.onDidChangeModelContent(function () {
        return handler(editorRef, monacoRef, diagnosticsProviders, decorationsProviders);
      });
      var definitionProviderDisposable = monacoRef.languages.registerDefinitionProvider('graphql', languageService.getDefinitionProvider(definitionProviders));
      var hoverDisposable = monacoRef.languages.registerHoverProvider('graphql', languageService.getHoverProvider(hoverProviders));
      return function () {
        hoverDisposable && hoverDisposable.dispose();
        definitionProviderDisposable && definitionProviderDisposable.dispose();
        onChangeDisposable && onChangeDisposable.dispose();
      };
    } // eslint-disable-next-line @typescript-eslint/no-empty-function


    return function () {};
  }, [editorRef, monacoRef]);
  return /*#__PURE__*/React.createElement(React.Fragment, null, /*#__PURE__*/React.createElement(MonacoEditor, _extends({
    height: '70vh'
  }, rest, {
    beforeMount: function beforeMount(monaco) {
      return setMonaco(monaco);
    },
    onMount: function onMount(editor) {
      return setEditor(editor);
    },
    onChange: function onChange(newValue) {
      return newValue && languageService.trySchema(newValue);
    },
    options: _objectSpread2({
      glyphMargin: true
    }, rest.options || {}),
    language: "graphql",
    defaultValue: schema
  })));
};

export { EnrichedLanguageService, SchemaEditor, coreDefinitionSource, coreDiagnosticsSource, coreHoverSource, debugHoverSource, showWidgetInPosition, toGraphQLPosition, toMarkerData, toMonacoRange };
//# sourceMappingURL=index.esm.js.map
